easypackages::packages('pheatmap', 'RColorBrewer', 'tidyverse')

csv <- "C:/MyPrograms/rna_seq_data_analysis/outputs/21-04-2023_11-19_list-genes_results.csv"

df <- read_csv(csv)

df_wide <- df %>% 
  pivot_wider(id_cols = c(group),
              names_from = species,
              values_from = c(starts_with('HL') | starts_with('LL'), gene_name, gene_id)) %>% 
  janitor::remove_empty(which = "cols")

df_means <- df_wide %>% 
  select(group, starts_with('LL'), starts_with('HL')) %>% 
  mutate(., 
       mean_At = rowMeans(select(., ends_with('thaliana')),
                                             na.rm = TRUE),
       mean_Br = rowMeans(select(., ends_with('rapa')),
                             na.rm = TRUE),
       mean_Bn = rowMeans(select(., ends_with('nigra')),
                             na.rm = TRUE),
       mean_Hi = rowMeans(select(., ends_with('incana')),
                             na.rm = TRUE),
       # mean_At_HL = rowMeans(select(df_wide, starts_with('HL') & ends_with('thaliana')),
       #                       na.rm = TRUE),
       # mean_Br_HL = rowMeans(select(df_wide, starts_with('HL') & ends_with('rapa')),
       #                       na.rm = TRUE),
       # mean_Bn_HL = rowMeans(select(df_wide, starts_with('HL') & ends_with('nigra')),
       #                       na.rm = TRUE),
       # mean_Hi_HL = rowMeans(select(df_wide, starts_with('HL') & ends_with('incana')),
       #                       na.rm = TRUE)
       ) %>% 
  #select(group, gene_name_A_thaliana, gene_id_A_thaliana, starts_with('mean'))
  select(group, starts_with('mean'))

# get rid of rows with zeros
df_means_clean <- df_means %>% 
  filter(if_all(starts_with('mean'), function(x) x > 0))

## - calculate ratios - ##

groups_ids <- df_means_clean %>% 
  pull(group)

df_means_clean_mat <- df_means_clean %>% 
  select(!group)

ratios <- do.call("cbind", 
                  lapply(seq_along(df_means_clean_mat), 
                                  function(y) apply(df_means_clean_mat, 
                                                    2, 
                                                    function(x) df_means_clean_mat[[y]]/x))) %>% 
  as.data.frame()

# rename ratios columns

col_names <- character()
ratio_names <- gsub("mean_", "", colnames(ratios)[1:4])

for(i in 1:length(ratio_names)){
  curr_name = ratio_names[i]
  for(nm in ratio_names){
    col_names <- append(col_names, paste0(curr_name, '/', nm))
  }
}

colnames(ratios) <- col_names

ratios_final_long <- ratios %>% 
  pivot_longer(cols = everything(),
               names_to = 'ratio',
               values_to = 'value')

## - make plots - ##
# not really useful at this point - plotting done individually below!

ratios_final_2 <- ratios %>% 
  mutate(`At/At` = NaN,
         `Br/Br` = NaN,
         `Bn/Bn` = NaN,
         `Hi/Hi` = NaN,)

ratios_final_long_2 <- ratios_final_2 %>% 
  pivot_longer(cols = everything(),
               names_to = 'ratio',
               values_to = 'value')

ratios_final_long_2 %>% 
  ggplot(aes(x=log2(value))) + 
  geom_hline(yintercept = 0)+
  geom_histogram(aes(y = after_stat(density)), binwidth = 0.1, fill = 'darkgrey') +
  geom_density(color = 'black', linewidth = 0.7) +
  geom_vline(xintercept = 0, color = 'red', linewidth = 0.7) +
  geom_vline(xintercept = 2, linetype = "dashed", color = 'blue', linewidth = 0.7) +
  geom_vline(xintercept = -2, linetype = "dashed", color = 'blue', linewidth = 0.7) +
  scale_x_continuous(limits = c(-8,8), breaks=scales::breaks_pretty(16)) +
  facet_wrap(.~ratio) +
  theme_grey(base_size = 16) +
  xlab('log2(ratio)') #+
  #theme(axis.text.x = element_text(size = 10))

## - calculate AUCs - ##

ratios_density <- ratios_final_long_2 %>% 
  filter(!is.na(value))

write_csv(ratios_density,
          file = 'outputs/ratios_exp_scc_density.csv')

ratios <- ratios_density %>% 
  pull(ratio) %>% 
  unique()

auc_tib <- tibble(ratio = character(),
                  peak = numeric(),
                  auc_l = numeric(),
                  auc_l_int = numeric(),
                  auc_h = numeric(),
                  auc_h_int = numeric())

#prep for plotting
#par(mfrow = c(3,4))

for(r in ratios){
  print(r)
  
  vals <- ratios_density %>% 
    filter(ratio == r) %>% 
    mutate(log2_value = log2(value)) %>% 
    pull(log2_value)
  
  d <- density.default(vals,
                       n = 512,
                       kernel = 'gaussian',
                       bw = 'nrd0',
                       cut = 10)
  
  xx <- d$x
  yy <- d$y
  
  #make a quick plot
  xx_max <- xx[which.max(yy)]
  
  png(file=paste0('outputs/', gsub('/', '_', r), '_density_plot.png'),
      width=500, height=400)

  plot(d, main = r, xlim = c(-8, 8),
       xlab = 'log2-ratio',
       cex.main=1.5, cex.lab=1.5, cex.axis=1,
       lwd = 1.75,
       xaxt="n")
  axis(side=1,at=seq(-8, 8, by = 1),cex = 1.5)
  abline(v = c(-1, 1), col = '#648FFF',lwd = 1.75) 
  abline(v = xx_max, col = '#DC267F',lwd = 1.75) 
  abline(v = 0, col = 'black', lwd = 1.75, lty = 3)
  
  dev.off()
  
  # estimate % AUC in the () interval
  # from https://stackoverflow.com/questions/40851328/compute-area-under-density-estimation-curve-i-e-probability
  f <- approxfun(xx, yy)
  C <- integrate(f, min(xx), max(xx))$value
  
  p1.unscaled <- integrate(f, min(xx), 0)$value
  p1.scaled <- p1.unscaled / C
  p2.unscaled <- integrate(f, 0, max(xx))$value
  p2.scaled <- p2.unscaled / C
  p3.unscaled <- integrate(f, -1, 0)$value
  p3.scaled <- p3.unscaled / C
  p4.unscaled <- integrate(f, 0, 1)$value
  p4.scaled <- p4.unscaled / C
  
  auc_tib <- bind_rows(auc_tib, tibble(ratio = r, peak = xx_max,
                                       auc_l = p1.scaled, auc_l_int = p3.scaled, 
                                       auc_h = p2.scaled, auc_h_int = p4.scaled))
}

write.csv(auc_tib,
          file = "outputs/auc_calculations.csv")
