# TPM_visualisation.R
# AUthor: Francesco Garassino

easypackages::packages('tidyverse')

stderr <- function(x) sd(x, na.rm = TRUE) / sqrt(length(x))

colors_treatments <- c('HL' = '#D41159', 'LL' = '#1A85FF')  
# https://davidmathlogic.com/colorblind/#%231A85FF-%23D41159

tpm_data_path <- "C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/26-04-2023_ath00196_TPM_counts.csv"
tpm_data <- read_csv(tpm_data_path, show_col_types = F)

## - LHCB8 story - ##
lhcb8_path <- "C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/LHCB8.csv"

lhcb8_data <- read_csv(lhcb8_path, show_col_types = F) 

lhcb8_data_plot <- lhcb8_data %>% 
  select(species, group, mean_LL, mean_HL) %>% 
  group_by(species, group) %>% 
  pivot_longer(cols = starts_with('mean'),
               names_to = 'treatment',
               names_prefix = 'mean_',
               values_to = 'mean_TPM') %>% 
  group_by(species, group, treatment) %>% 
  summarise(avg_tpm_copies = mean(mean_TPM)) %>%
  mutate(gene = case_when(
    group == 921162 ~ 'LHCB4',
    group == 928236 ~ 'LHCB5',
    group == 923995 ~ 'LHCB6',
    group == 921163 ~ 'LHCB8'
  )) %>% 
  mutate(group = factor(group))

a <- lhcb8_data_plot %>% 
  #filter(species == 'H_incana') %>% 
  group_by(species, treatment) %>% 
  arrange(treatment) %>% 
  arrange(species)

b <- lhcb8_data_plot %>% 
  #filter(species == 'H_incana') %>% 
  group_by(species, treatment) %>% 
  reframe(proportion = avg_tpm_copies/sum(avg_tpm_copies))

lhcb8_proportions <- bind_cols(a, b) %>% 
  dplyr::select("species...1", "group", "treatment...3", "avg_tpm_copies",
         "gene", "proportion") %>% 
  janitor::clean_names() %>% 
  dplyr::rename(species = species_1, treatment = treatment_3) %>% 
  mutate(treatment = factor(treatment, levels = c('LL', 'HL')),
         percentage = 100*proportion) %>% 
  mutate(percentage_hr = paste0(round(percentage,1), '%'))

cp <- coord_polar(theta = "y")
cp$is_free <- function() TRUE

fourcols_pal <- c("#56B4E9", "#009E73", "#D55E00", "#CC79A7")

lhcb8_pie_fig <- lhcb8_proportions %>% 
  filter(species == 'A_thaliana' | species == 'H_incana') %>%
  unite(species_treatment, species, treatment, sep = '-') %>% 
  mutate(species_treatment = factor(species_treatment, 
                                    levels = c('A_thaliana-LL', 'A_thaliana-HL',
                                               'H_incana-LL', 'H_incana-HL'))) %>% 
  ggplot(aes(x = '', y = percentage, fill = gene)) +
  geom_bar(stat="identity", width=1) +
  cp +
  geom_text(aes(label = percentage_hr),
           position = position_stack(vjust = 0.3)) +
  scale_fill_manual(values = fourcols_pal) +
  facet_wrap(. ~species_treatment, ncol = 2, scales = 'free') +
  theme_grey(base_size = 14) +
  theme(aspect.ratio = 1,
        axis.text = element_blank(),
        axis.ticks = element_blank(),
        panel.grid  = element_blank(),
        axis.title = element_blank(),
        legend.position = 'bottom',
        legend.title = element_blank()) 

lhcb8_pie_suppfig <- lhcb8_proportions %>% 
  filter(species == 'B_rapa' | species == 'B_nigra') %>%
  unite(species_treatment, species, treatment, sep = '-') %>% 
  mutate(species_treatment = factor(species_treatment, 
                                    levels = c('B_rapa-LL', 'B_rapa-HL',
                                               'B_nigra-LL', 'B_nigra-HL'))) %>% 
  ggplot(aes(x = '', y = percentage, fill = gene)) +
  geom_bar(stat="identity", width=1) +
  cp +
  geom_text(aes(label = percentage_hr),
            position = position_stack(vjust = 0.5)) +
  scale_fill_manual(values = fourcols_pal) +
  facet_wrap(.~species_treatment, ncol = 2, scales = 'free') +
  theme_grey(base_size = 14) +
  theme(aspect.ratio = 1,
        axis.text = element_blank(),
        axis.ticks = element_blank(),
        panel.grid  = element_blank(),
        axis.title = element_blank(),
        legend.position = 'bottom',
        legend.title = element_blank())
  

## - LHCA6 story - ##

lhca6_path <- "C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/LHCA6.csv"

lhca6_data <- read_csv(lhca6_path, show_col_types = F) 

# ANOVA between species

lhca6_data_sum <- lhca6_data %>% 
  mutate(species = factor(species, 
                          levels = c('A_thaliana', 'B_rapa', 'B_nigra', 'H_incana')),
         copy = factor(copy)) %>% 
  dplyr::select(species, copy, starts_with(c('LL', 'HL'))) %>% 
  group_by(species) %>% 
  summarise(across(starts_with(c('LL', 'HL')), sum)) 

lhca6_data_aov <- lhca6_data_sum %>% 
  pivot_longer(cols = starts_with(c('LL', 'HL')),
               names_to = 'treatment',
               values_to = 'cumulative_tpm') %>% 
  mutate(treatment = as.factor(gsub('\\d', '', treatment)))

res_aov_lhca6 <- aov(cumulative_tpm ~ species*treatment, data = lhca6_data_aov)
summary(res_aov_lhca6)
tukey_lhca6 <- glht(res_aov_lhca6, linfct=mcp(species="Tukey"))
cld_lhca6 <- cld(tukey_lhca6)

# making the bar plot
lhca6_data_means <- lhca6_data_sum %>% 
  mutate(mean_LL = rowMeans(dplyr::select(., starts_with('LL')), na.rm = TRUE),
         sd_LL = apply(dplyr::select(., starts_with('LL')), 1, sd, na.rm=TRUE),
         stderr_LL = apply(dplyr::select(., starts_with('LL')), 1, stderr),
         mean_HL = rowMeans(dplyr::select(., starts_with('HL')), na.rm = TRUE),
         sd_HL = apply(dplyr::select(., starts_with('HL')), 1, sd, na.rm=TRUE),
         stderr_HL = apply(dplyr::select(., starts_with('HL')), 1, stderr)) %>% 
  dplyr::select(!starts_with(c('LL', 'HL')))

ll <- lhca6_data_means %>% 
  dplyr::select(species, ends_with('_LL')) %>% 
  setNames(c('species','mean','sd','stderr')) %>% 
  mutate(treatment = 'LL')

hl <- lhca6_data_means %>% 
  dplyr::select(species, ends_with('_HL')) %>% 
  setNames(c('species','mean','sd','stderr')) %>% 
  mutate(treatment = 'HL')

lhca6_data_means_plot <- bind_rows(ll, hl) %>%
  mutate(species = factor(species, 
                          levels = c('A_thaliana', 'B_rapa', 'B_nigra', 'H_incana'))) %>%
  arrange(species) %>% 
  unite('species_treatment', c(species, treatment), sep = '-') %>% 
  mutate(species_treatment = factor(species_treatment, 
                                    levels = unique(.$species_treatment)))

lhca6_data_means_plot %>% 
  ggplot(aes(species_treatment, mean)) +
  geom_bar(fill = '#648FFF', stat="identity") +
  geom_errorbar(aes(ymin = mean - stderr, ymax = mean + stderr), width = 0.2) +
  theme_grey(base_size = 16) +
  theme(axis.title.x = element_blank(),
        axis.text.x = element_text(angle = 45, hjust = 1)) +
  scale_y_continuous(limits = c(0,1000), breaks = scales::breaks_pretty(10)) +
  ylab('mean TPM counts')


lhca6_data_long <- lhca6_data %>% 
  mutate(species = factor(species, 
                          levels = c('A_thaliana', 'B_rapa', 'B_nigra', 'H_incana')),
         copy = factor(copy)) %>% 
  dplyr::select(species, copy, log2FC, padj, signif, mean_LL, mean_HL) %>% 
  pivot_longer(cols = starts_with('mean'),
                               names_to = 'treatment',
                               names_prefix = 'mean_',
                               values_to = 'mean_TPM') %>% 
  mutate(treatment = factor(treatment, levels = unique(treatment))) %>% 
  arrange(species, treatment) 

lhca6_data_unite <- lhca6_data_long %>%
  unite('species_treatment', c(species, treatment), sep = '-')

lhca6_data_plot <- lhca6_data_unite %>% 
  mutate(species_treatment = factor(species_treatment, levels = unique(lhca6_data_unite$species_treatment))) %>% 
  mutate(copy = factor(copy, 
                       levels = rev(unique((lhca6_data_unite$copy)))))  %>%
  left_join(lhca6_data_means_plot, by = 'species_treatment')

colors_copies = c('#FFB000', '#DC267F', '#648FFF')
# https://davidmathlogic.com/colorblind/#%23648FFF-%23785EF0-%23DC267F-%23FE6100-%23FFB000

lhca6_p <- lhca6_data_plot %>% 
  ggplot(aes(x = species_treatment, y = mean_TPM, fill = copy)) +
  geom_bar(stat="identity", position='stack') +
  scale_fill_manual(values = colors_copies) +
  theme_grey(base_size = 16) +
  theme(axis.title.x = element_blank(),
        axis.text.x = element_text(angle = 45, hjust = 1),
        legend.position = 'none') +
  scale_y_continuous(limits = c(0,1000), breaks = scales::breaks_pretty(10)) +
  ylab('mean TPM counts')

lhca6_p

lhca6_data_sum <- lhca6_data_long %>% 
  group_by(species, treatment) %>% 
  summarise(sum_mean_TPM = sum(mean_TPM))

ll <- lhca6_data_sum %>% 
  filter(treatment == 'LL') %>% 
  dplyr::select(species, sum_mean_TPM) %>% 
  pivot_wider(names_from = species,
              values_from = sum_mean_TPM)

ratios_ll <- do.call("cbind", 
                  lapply(seq_along(ll), 
                         function(y) apply(ll, 
                                           2, 
                                           function(x) log2(ll[[y]]/x)))) %>% 
  as.data.frame()
colnames(ratios_ll) <- rownames(ratios_ll)

write.csv(ratios_ll, 
          file = 'outputs/LHCA6_ratios_LL.csv')

hl <- lhca6_data_sum %>% 
  filter(treatment == 'HL') %>% 
  select(species, sum_mean_TPM) %>% 
  pivot_wider(names_from = species,
              values_from = sum_mean_TPM)

ratios_hl <- do.call("cbind", lapply(seq_along(hl), function(y) apply(hl, 2, function(x) log2(hl[[y]]/x)))) %>% 
  as.data.frame()

colnames(ratios_hl) <- rownames(ratios_hl)

write.csv(ratios_hl, 
          file = 'outputs/LHCA6_ratios_HL.csv')  

## - PSA/PSB story- ##

ps_path <- "C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/PSA-PSB.csv"

ps_data <- read_csv(ps_path, show_col_types = F) 

ps_data_means <- ps_data %>% 
  mutate(gene = case_when(
    group == 108 ~ 'PSAA',
    group == 388 ~ 'PSAJ',
    group == 103 ~ 'PSBD',
    group == 385 ~ 'PSBE'
  )) %>%
  mutate(mean_LL = rowMeans(dplyr::select(., starts_with('LL')), na.rm = TRUE),
         sd_LL = apply(dplyr::select(., starts_with('LL')), 1, sd, na.rm=TRUE),
         stderr_LL = apply(dplyr::select(., starts_with('LL')), 1, stderr),
         mean_HL = rowMeans(dplyr::select(., starts_with('HL')), na.rm = TRUE),
         sd_HL = apply(dplyr::select(., starts_with('HL')), 1, sd, na.rm=TRUE),
         stderr_HL = apply(dplyr::select(., starts_with('HL')), 1, stderr)) %>% 
  dplyr::select(!starts_with(c('LL', 'HL')))

ll_ps <- ps_data_means %>% 
  dplyr::select(species, gene, ends_with('_LL')) %>% 
  setNames(c('species','gene','mean','sd','stderr')) %>% 
  mutate(treatment = 'LL')

hl_ps <- ps_data_means %>% 
  dplyr::select(species, gene, ends_with('_HL')) %>% 
  setNames(c('species','gene','mean','sd','stderr')) %>% 
  mutate(treatment = 'HL')

ps_data_plot <- bind_rows(ll_ps, hl_ps) %>%
  mutate(species = factor(species, 
                          levels = c('A_thaliana', 'B_rapa', 'B_nigra', 'H_incana'))) %>%
  arrange(species) %>% 
  unite('species_treatment', c(species, treatment), sep = '-') %>% 
  mutate(species_treatment = factor(species_treatment, 
                                    levels = unique(.$species_treatment)),
         gene = factor(gene, levels = c('PSBD', 'PSBE', 'PSAA', 'PSAJ')))

ps_facet_plot <- ps_data_plot %>% 
  ggplot(aes(species_treatment, mean)) +
  geom_bar(fill = '#648FFF', stat="identity") +
  geom_errorbar(aes(ymin = mean - stderr, ymax = mean + stderr), width = 0.2) +
  theme_grey(base_size = 16) +
  theme(axis.title.x = element_blank(),
        plot.title = element_text(face = 'bold')) +
  ylab('mean TPM counts') +
  facet_wrap(.~gene, scales = 'free_y') +
  theme_grey(base_size = 14) +
  theme(axis.text.x = element_text(angle = 45, hjust = 1),
        axis.title.x = element_blank())

#ratios
all_ratios_ll = list()
all_ratios_hl = list()
i = 0
for(g in unique(ps_data_means %>% dplyr::pull(gene))){
  i <- i + 1
  print(i)
  print(g)

ll_curr <- ll_ps %>% filter(gene == g) %>%  
  dplyr::select(species, mean) %>% 
  pivot_wider(values_from = mean, names_from = species)

ratios_ll <- do.call("cbind", 
                     lapply(seq_along(ll_curr), 
                            function(y) apply(ll_curr, 
                                              2, 
                                              function(x) log2(ll_curr[[y]]/x)))) %>% 
  as.data.frame()
colnames(ratios_ll) <- rownames(ratios_ll)

all_ratios_ll <- append(all_ratios_ll, list(ratios_ll))

hl_curr <- hl_ps %>% filter(gene == g) %>%  
  dplyr::select(species, mean) %>% 
  pivot_wider(values_from = mean, names_from = species)

ratios_hl <- do.call("cbind", lapply(seq_along(hl_curr), 
                                     function(y) apply(hl_curr, 
                                                       2, 
                                                       function(x) log2(hl_curr[[y]]/x))))
colnames(ratios_hl) <- rownames(ratios_hl)
all_ratios_hl <- append(all_ratios_hl, list((ratios_hl)))
}

write.csv(all_ratios_ll, 
          file = 'outputs/PSA-PSB_ratios_LL.csv')  

write.csv(all_ratios_hl, 
          file = 'outputs/PSA-PSB_ratios_HL.csv')  

## - making a single figure - ##
# 
# cowplot::plot_grid(lhcb8_pie_fig, lhca6_p, ps_facet_plot,
#                    labels = 'auto')
