# ratios_dist_plots_targeted.R
# Author: Francesco Garassino

# Script used for plotting ratios for selected genes and calculating AUCs respective to 
# non-DE scc genes

ratios_density <- read_csv('outputs/ratios_exp_scc_density.csv')

ratios <- ratios_density %>% 
  pull(ratio) %>% 
  unique()

ratios_sel <- ratios[c(4,7,8,10,11,12)]

## - LHCA6 - ##

ratios_lhca6 <- read_csv('C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/LHCA6_ratios.csv')

auc_tib <- tibble(ratio = character(),
                  auc_left_LL = numeric(),
                  auc_right_LL = numeric(),
                  auc_left_HL = numeric(),
                  auc_right_HL = numeric())

#prep for plotting
#par(mfrow = c(3,4))

for(r in ratios_sel){
  print(r)
  
  vals <- ratios_density %>% 
    filter(ratio == r) %>% 
    mutate(log2_value = log2(value)) %>% 
    pull(log2_value)
  
  d <- density.default(vals,
                       n = 512,
                       kernel = 'gaussian',
                       bw = 'nrd0',
                       cut = 10)
  
  xx <- d$x
  yy <- d$y
  
  #make a quick plot
  #xx_max <- xx[which.max(yy)]
  x_LL <- ratios_lhca6 %>% filter(ratio == r & treatment == 'LL') %>% 
    pull(value)
  x_HL <-  ratios_lhca6 %>% filter(ratio == r & treatment == 'HL') %>% 
    pull(value)
  
  png(file=paste0('outputs/', gsub('/', '_', r), '_lhca6_ratio_dplot.png'),
      width=500, height=400)
  
  plot(d, main = r, xlim = c(-8, 8),
       xlab = 'log2-ratio',
       cex.main=1.5, cex.lab=1.5, cex.axis=1,
       lwd = 1.75,
       xaxt="n")
  axis(side=1,at=seq(-8, 8, by = 1),cex = 1.5)
  #abline(v = c(-1, 1), col = '#648FFF',lwd = 1.75) 
  #abline(v = xx_max, col = '#DC267F',lwd = 1.75) 
  abline(v = 0, col = 'black', lwd = 1.75, lty = 3)
  abline(v = x_LL, col = '#648FFF', lwd = 1.75)
  abline(v = x_HL, col = '#DC267F', lwd = 1.75)
  
  dev.off()
  
  # estimate % AUC in the () interval
  # from https://stackoverflow.com/questions/40851328/compute-area-under-density-estimation-curve-i-e-probability
  f <- approxfun(xx, yy)
  C <- integrate(f, min(xx), max(xx), subdivisions=200)$value
  
  p1.unscaled <- integrate(f, min(xx), x_LL, subdivisions=200)$value
  p1.scaled <- p1.unscaled / C
  p2.unscaled <- integrate(f, x_LL, max(xx), subdivisions=200)$value
  p2.scaled <- p2.unscaled / C
  p3.unscaled <- integrate(f, min(xx), x_HL, subdivisions=200)$value
  p3.scaled <- p3.unscaled / C
  p4.unscaled <- integrate(f, x_HL, max(xx), subdivisions=200)$value
  p4.scaled <- p4.unscaled / C
  
  auc_tib <- bind_rows(auc_tib, tibble(ratio = r,
                                       auc_left_LL = p1.scaled, auc_right_LL = p2.scaled, 
                                       auc_left_HL = p3.scaled, auc_right_HL = p4.scaled))
}
write.csv(auc_tib,
          file = "outputs/LHCA6_auc_calculations.csv")

## - PSB-PSA - ##

ratios_psa_psb <- read_csv('C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/PSA-PSB_ratios.csv')

genes <- ratios_psa_psb %>% 
  pull(gene) %>% 
  unique()

for(g in genes){
  print(g)
  
  auc_tib <- tibble(ratio = character(),
                    auc_left_LL = numeric(),
                    auc_right_LL = numeric(),
                    auc_left_HL = numeric(),
                    auc_right_HL = numeric())
  
  for(r in ratios_sel){
    print(r)
    
    vals <- ratios_density %>% 
      filter(ratio == r) %>% 
      mutate(log2_value = log2(value)) %>% 
      pull(log2_value)
    
    d <- density.default(vals,
                         n = 512,
                         kernel = 'gaussian',
                         bw = 'nrd0',
                         cut = 10)
    
    xx <- d$x
    yy <- d$y
    
    #make a quick plot
    #xx_max <- xx[which.max(yy)]
    x_LL <- ratios_psa_psb %>% filter(ratio == r & gene == g & treatment == 'LL') %>% 
      pull(value)
    x_HL <-  ratios_psa_psb %>% filter(ratio == r & gene == g & treatment == 'HL') %>% 
      pull(value)
    
    png(file=paste0('outputs/', gsub('/', '_', r), '_', g, '_ratio_dplot.png'),
        width=500, height=400)
    
    plot(d, main = r, xlim = c(-8, 8),
         xlab = 'log2-ratio',
         cex.main=1.5, cex.lab=1.5, cex.axis=1,
         lwd = 1.75,
         xaxt="n")
    axis(side=1,at=seq(-8, 8, by = 1),cex = 1.5)
    #abline(v = c(-1, 1), col = '#648FFF',lwd = 1.75) 
    #abline(v = xx_max, col = '#DC267F',lwd = 1.75) 
    abline(v = 0, col = 'black', lwd = 1.75, lty = 3)
    abline(v = x_LL, col = '#648FFF', lwd = 1.75)
    abline(v = x_HL, col = '#DC267F', lwd = 1.75)
    
    dev.off()
    
    # estimate % AUC in the () interval
    # from https://stackoverflow.com/questions/40851328/compute-area-under-density-estimation-curve-i-e-probability
    f <- approxfun(xx, yy)
    C <- integrate(f, min(xx), max(xx), subdivisions=200, rel.tol=.Machine$double.eps^.05)$value
    
    p1.unscaled <- integrate(f, min(xx), x_LL, subdivisions=200, rel.tol=.Machine$double.eps^.05)$value
    p1.scaled <- p1.unscaled / C
    p2.unscaled <- integrate(f, x_LL, max(xx), subdivisions=200, rel.tol=.Machine$double.eps^.05)$value
    p2.scaled <- p2.unscaled / C
    p3.unscaled <- integrate(f, min(xx), x_HL, subdivisions=200, rel.tol=.Machine$double.eps^.05)$value
    p3.scaled <- p3.unscaled / C
    p4.unscaled <- integrate(f, x_HL, max(xx), subdivisions=200, rel.tol=.Machine$double.eps^.05)$value
    p4.scaled <- p4.unscaled / C
    
    auc_tib <- bind_rows(auc_tib, tibble(ratio = r,
                                         auc_left_LL = p1.scaled, auc_right_LL = p2.scaled, 
                                         auc_left_HL = p3.scaled, auc_right_HL = p4.scaled))
  }
  write.csv(auc_tib,
            file = paste0("outputs/", g, "_auc_calculations.csv"))
}
