## upset_plot_ortho.R
## author: Dirk-Jan van Workum, Francesco Garassino

## allows to prepare upset plots for the panproteome

# Load packages
library(ComplexUpset)
library(ggplot2)

setwd("C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/3_Pan-proteome/upset_plot")

# Get data
names <- c("A. arabicum", "A. thaliana", "B. nigra", "B. rapa", "H. incana", "R. raphanistrum", "R. sativus", "S. irio")
rawData <- read.csv("classified_groups.csv", header = T)
colnames(rawData)[c(3:(ncol(rawData) - 1))] <- names
rawData[which(rawData[, 2] == "core & single copy orthologous"), 2] <- "core"

# Plot with complex upset - FILTERED
complexData <- rawData
complexData[names] <- complexData[names] != 0
complexData$class <- factor(as.factor(complexData$class), levels = c("unique", "accessory", "core"))

p1 <- upset(
  complexData,
  names,
  #n_intersections=15,
  min_size=200,
  set_sizes = (
    upset_set_size(
      geom = geom_bar(
        aes(fill = class),
        width = 0.6
      ),
      position='right'
    )
  ),
  base_annotations=list(
    'Intersection size'=intersection_size(
      text=list(
        vjust=-0.1,
        hjust=-0.1,
        angle=45,
        color = 'black'
      )
    )
  )
)

# Plot with four-species upset
lesscomplexData <- rawData[1:7]
lesscomplexData = lesscomplexData[-c(3)]
names_4 = names[2:5]

lesscomplexData[names_4] <- lesscomplexData[names_4] != 0
lesscomplexData$class <- factor(as.factor(lesscomplexData$class), levels = c("unique", "accessory", "core"))

upset(
  lesscomplexData,
  names_4,
  set_sizes = (
    upset_set_size(
      geom = geom_bar(
        aes(fill = class),
        width = 0.6
      ),
      position='right'
    )
  ),
  base_annotations=list(
    'Intersection size'=intersection_size(
      text=list(
        vjust=-0.1,
        hjust=-0.1,
        angle=45,
        color = 'black')
      )
    )
)
