# heatmap_list_genes.R
# author: Francesco Garassino

# given a DE table (produced with analyse_orthology*.py), allows to plot a series of 
# heatmaps


easypackages::packages('tidyverse', 'pheatmap', 'RColorBrewer')


## - This is about number of DE copies - ##

#data_path <- "C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/29-03-2023_ath00195_DE_table.csv"
#data_path <- "C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/30-03-2023_ath01200_DE_table.csv"
data_path <- "C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/04-04-2023_kegg_ath_M00165_DE_table.csv"


data <- read_csv(data_path)

data_hmap <- data %>% 
  select(matches('de_copies_.*')) %>% 
  as.matrix()

gene_names <- data %>%
  pull(gene_name_At)

groups <- data %>% 
  pull(group_id)

colnames(data_hmap) <- c("A_thaliana", "B_nigra", "B_rapa", "H_incana")
new_colnames <- c("A_thaliana", "B_rapa", "B_nigra", "H_incana")
data_hmap <- data_hmap[,new_colnames]

hmap <- pheatmap(data_hmap,
                 color = colorRampPalette(rev(brewer.pal(n = 7, name ="RdYlBu")))(100),
                 kmeans_k = NA,
                 scale = "none",
                 cluster_rows = TRUE,
                 cluster_cols = FALSE,
                 clustering_distance_rows = "euclidean",
                 clustering_distance_cols = "euclidean",
                 clustering_method = "complete",
                 labels_row = gsub("'", "", gene_names),
                 #labels_row = groups,
                 na_col = "grey50")

## - Let's do something with log2FC - ##
## these are the actual supp figures

# ath00196
data2_path <- "C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/26-04-2023_ath00196_TPM_counts.csv"

# ath00195
#data2_path <- "C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/29-03-2023_ath00195_TPM_counts.csv"

#ath01200
#data2_path <- "C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/30-03-2023_ath01200_TPM_counts.csv"

#ath_M00165
#data2_path <- "C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/04-04-2023_kegg_ath_M00165_TPM_counts.csv"

data2 <- read_csv(data2_path)

gene_name_pairs <- data2 %>% 
  filter(group != 906836) %>% 
  filter(species == "A_thaliana") %>% 
  select(gene_id, group, gene_name) %>% 
  group_by(group) %>% 
  summarise(gene_names = toupper(paste(gene_name, collapse = ", ")))



data4_hmap_tib <- data2 %>% 
  filter(group != 906836) %>% 
  select(species, gene_id, group, log2FC, signif) %>% 
  mutate(log2FC = case_when(signif == "NS" ~ NA,
                            signif != "NS" ~ log2FC)) %>% 
  group_by(group, species) %>% 
  summarise(n_genes = n(),
            avg_log2FC = mean(log2FC, na.rm = TRUE)) %>% 
  select(group, species, avg_log2FC) %>% 
  pivot_wider(names_from = species,
              values_from = avg_log2FC) %>% 
  arrange(group)

gene_names4 <- read_csv("C:/Users/garas001/OneDrive - Wageningen University & Research/PhD/_Year 4/Chapter 3 - RNA-Seq Hirschfeldia/_Chapter writing/data_last version/29-03-2023_ath00195_DE_table.csv") %>% 
  arrange(group_id) %>% 
  pull(gene_name_At) %>% 
  gsub("\\['", '', .) %>% 
  gsub("'\\]", '', .)
 
data4_hmap <- data4_hmap_tib %>% 
  #filter(group != 943185) %>% 
  ungroup() %>% 
  select(!group) %>%
  as.matrix()

colnames(data4_hmap) <- c("A_thaliana", "B_nigra", "B_rapa", "H_incana")
new_colnames <- c("A_thaliana", "B_rapa", "B_nigra", "H_incana")
data4_hmap <- data4_hmap[,new_colnames]

col_names <- c("A. thaliana", "B. rapa", "B. nigra", "H. incana")
#data4_hmap[is.nan(data4_hmap)] <- 0

gene_names4 <- gene_name_pairs %>%
  pull(gene_names) %>% 
  gsub('%[25]+', '', .)

paletteLength <- 50
myColor <- colorRampPalette(rev(brewer.pal(n = 7, name ="RdYlBu")))(paletteLength)
# length(breaks) == length(paletteLength) + 1
# use floor and ceiling to deal with even/odd length palette lengths
myBreaks <- c(seq(min(data4_hmap, na.rm = T), 0, length.out=ceiling(paletteLength/2) + 1), 
              seq(max(data4_hmap, na.rm = T)/paletteLength, max(data4_hmap, na.rm = T), length.out=floor(paletteLength/2)))


hmap4 <- pheatmap(data4_hmap[-68,],
                  #color = colorRampPalette(rev(brewer.pal(n = 7, name ="RdYlBu")))(100),
                  color = myColor,
                  breaks = myBreaks,
                  kmeans_k = NA,
                  scale = "none",
                  cluster_rows = FALSE,
                  cluster_cols = FALSE,
                  clustering_distance_rows = "euclidean",
                  clustering_distance_cols = "euclidean",
                  clustering_method = "complete",
                  #labels_row = toupper(gsub("'", "", gene_names4[-68])),
                  labels_row = gsub("'", "", gene_names4),
                  labels_col = col_names,
                  na_col = "grey50",
                  fontsize = 14,
                  angle_col = 45)
  